@@ -12,13 +12,14 @@
 #include <iostream>
 #include <vector>
 
-#include "zypp/base/Logger.h"
+#include "zypp/base/LogTools.h"
 #include "zypp/base/DefaultIntegral.h"
 #include "zypp/parser/xml/XmlEscape.h"
 
 #include "zypp/RepoInfo.h"
 #include "zypp/TriBool.h"
 #include "zypp/Pathname.h"
+#include "zypp/ZConfig.h"
 #include "zypp/repo/RepoMirrorList.h"
 #include "zypp/ExternalProgram.h"
 #include "zypp/media/MediaAccess.h"
@@ -42,8 +43,12 @@ namespace zypp
   struct RepoInfo::Impl
   {
     Impl()
-      : gpgcheck(indeterminate)
+      : _gpgCheck( indeterminate )
+      , _repoGpgCheck( indeterminate )
+      , _pkgGpgCheck( indeterminate )
+      , _validRepoSignature( indeterminate )
       ,	keeppackages(indeterminate)
+      , _mirrorListForceMetalink(false)
       , type(repo::RepoType::NONE_e)
       , emptybaseurls(false)
     {}
@@ -68,34 +73,20 @@ namespace zypp
     Pathname licenseTgz() const
     { return metadatapath.empty() ? Pathname() : metadatapath / path / "license.tar.gz"; }
 
-    Url getmirrorListUrl() const
-    { return replacer(mirrorlist_url); }
-
-    Url &setmirrorListUrl()
-    { return mirrorlist_url; }
-
-    const std::set<Url> &baseUrls() const
+    const RepoVariablesReplacedUrlList & baseUrls() const
     {
-      if ( _baseUrls.empty() && ! (getmirrorListUrl().asString().empty()) )
+      const Url & mlurl( _mirrorListUrl.transformed() );	// Variables replaced!
+      if ( _baseUrls.empty() && ! mlurl.asString().empty() )
       {
         emptybaseurls = true;
-        repo::RepoMirrorList *rmirrorlist = NULL;
-
         DBG << "MetadataPath: " << metadatapath << endl;
-        if( metadatapath.empty() )
-          rmirrorlist = new repo::RepoMirrorList (getmirrorListUrl() );
-        else
-          rmirrorlist = new repo::RepoMirrorList (getmirrorListUrl(), metadatapath );
-
-        std::vector<Url> rmurls = rmirrorlist->getUrls();
-        delete rmirrorlist;
-        rmirrorlist = NULL;
-        _baseUrls.insert(rmurls.begin(), rmurls.end());
+	repo::RepoMirrorList rmurls( mlurl, metadatapath, _mirrorListForceMetalink );
+	_baseUrls.raw().insert( _baseUrls.raw().end(), rmurls.getUrls().begin(), rmurls.getUrls().end() );
       }
       return _baseUrls;
     }
 
-    std::set<Url> &baseUrls()
+    RepoVariablesReplacedUrlList & baseUrls()
     { return _baseUrls; }
 
     bool baseurl2dump() const
@@ -148,13 +139,77 @@ namespace zypp
 	/////////////////////////////////////////////////////////////////
       }
       return( _keywords.find( keyword_r ) != _keywords.end() );
-
     }
 
+    /** Signature check result needs to be stored/retrieved from _metadatapath.
+     * Don't call them from outside validRepoSignature/setValidRepoSignature
+     */
+    //@{
+    TriBool internalValidRepoSignature() const
+    {
+      if ( ! indeterminate(_validRepoSignature) )		return _validRepoSignature;
+      // check metadata:
+      if ( ! metadatapath.empty() )
+      {
+	//TODO: a missing ".repo_gpgcheck" might be plaindir(no Downloader) or not yet refreshed signed repo!
+	TriBool linkval = triBoolFromPath( metadatapath / ".repo_gpgcheck" );
+	return linkval;
+      }
+      return indeterminate;
+    }
+
+    void internalSetValidRepoSignature( TriBool value_r )
+    {
+      if ( PathInfo(metadatapath).isDir() )
+      {
+	Pathname gpgcheckFile( metadatapath / ".repo_gpgcheck" );
+	if ( PathInfo(gpgcheckFile).isExist() )
+	{
+	  TriBool linkval( indeterminate );
+	  if ( triBoolFromPath( gpgcheckFile, linkval ) && linkval == value_r )
+	    return;	// existing symlink fits value_r
+	  else
+	    filesystem::unlink( gpgcheckFile );	// will write a new one
+	}
+	filesystem::symlink( asString(value_r), gpgcheckFile );
+      }
+      _validRepoSignature = value_r;
+    }
+
+    bool triBoolFromPath( const Pathname & path_r, TriBool & ret_r ) const
+    {
+      static const Pathname truePath( "true" );
+      static const Pathname falsePath( "false" );
+      static const Pathname indeterminatePath( "indeterminate" );
+      Pathname linkval( filesystem::readlink( path_r ) );
+      bool known = true;
+      if ( linkval == truePath )
+	ret_r = true;
+      else if ( linkval == falsePath )
+	ret_r = false;
+      else if ( linkval == indeterminatePath )
+	ret_r = indeterminate;
+      else
+	known = false;
+      return known;
+    }
+
+    TriBool triBoolFromPath( const Pathname & path_r ) const
+    { TriBool ret(indeterminate); triBoolFromPath( path_r, ret ); return ret; }
+
+    //@}
+
+  public:
+    TriBool _gpgCheck;		///< default gpgcheck behavior: Y/N/ZConf
+    TriBool _repoGpgCheck;	///< need to check repo sign.: Y/N/(ZConf(Y/N/gpgCheck))
+    TriBool _pkgGpgCheck;	///< need to check pkg sign.: Y/N/(ZConf(Y/N/gpgCheck && no valid repo sign.))
+  private:
+    TriBool _validRepoSignature;///< have  signed and valid repo metadata
   public:
-    TriBool gpgcheck;
     TriBool keeppackages;
-    Url gpgkey_url;
+    RepoVariablesReplacedUrl _gpgKeyUrl;
+    RepoVariablesReplacedUrl _mirrorListUrl;
+    bool                     _mirrorListForceMetalink;
     repo::RepoType type;
     Pathname path;
     std::string service;
@@ -166,8 +221,7 @@ namespace zypp
     repo::RepoVariablesUrlReplacer replacer;
 
   private:
-    Url mirrorlist_url;
-    mutable std::set<Url> _baseUrls;
+    mutable RepoVariablesReplacedUrlList _baseUrls;
     mutable std::set<std::string> _keywords;
 
     friend Impl * rwcowClone<Impl>( const Impl * rhs );
@@ -191,24 +245,12 @@ namespace zypp
 
   const RepoInfo RepoInfo::noRepo;
 
-  ///////////////////////////////////////////////////////////////////
-  //
-  //	METHOD NAME : RepoInfo::RepoInfo
-  //	METHOD TYPE : Ctor
-  //
   RepoInfo::RepoInfo()
   : _pimpl( new Impl() )
   {}
 
-  ///////////////////////////////////////////////////////////////////
-  //
-  //	METHOD NAME : RepoInfo::~RepoInfo
-  //	METHOD TYPE : Dtor
-  //
   RepoInfo::~RepoInfo()
-  {
-    //MIL << std::endl;
-  }
+  {}
 
   unsigned RepoInfo::priority() const
   { return _pimpl->priority; }
@@ -219,22 +261,74 @@ namespace zypp
   void RepoInfo::setPriority( unsigned newval_r )
   { _pimpl->priority = newval_r ? newval_r : Impl::defaultPriority; }
 
-  void RepoInfo::setGpgCheck( bool check )
-  { _pimpl->gpgcheck = check; }
 
-  void RepoInfo::setMirrorListUrl( const Url &url )
-  { _pimpl->setmirrorListUrl() = url; }
+  bool RepoInfo::gpgCheck() const
+  { return indeterminate(_pimpl->_gpgCheck) ? ZConfig::instance().gpgCheck() : (bool)_pimpl->_gpgCheck; }
 
-  void RepoInfo::setGpgKeyUrl( const Url &url )
-  { _pimpl->gpgkey_url = url; }
+  void RepoInfo::setGpgCheck( TriBool value_r )
+  { _pimpl->_gpgCheck = value_r; }
 
-  void RepoInfo::addBaseUrl( const Url &url )
-  { _pimpl->baseUrls().insert(url); }
+  void RepoInfo::setGpgCheck( bool value_r ) // deprecated legacy and for squid
+  { setGpgCheck( TriBool(value_r) ); }
 
-  void RepoInfo::setBaseUrl( const Url &url )
+
+  bool RepoInfo::repoGpgCheck() const
   {
-    _pimpl->baseUrls().clear();
-    addBaseUrl(url);
+    if ( ! indeterminate(_pimpl->_repoGpgCheck) )		return _pimpl->_repoGpgCheck;
+    if ( ! indeterminate(ZConfig::instance().repoGpgCheck()) )	return ZConfig::instance().repoGpgCheck();
+    return gpgCheck();	// no preference: follow gpgCheck
+  }
+
+  void RepoInfo::setRepoGpgCheck( TriBool value_r )
+  { _pimpl->_repoGpgCheck = value_r; }
+
+
+  bool RepoInfo::pkgGpgCheck() const
+  {
+    if ( ! indeterminate(_pimpl->_pkgGpgCheck) )		return _pimpl->_pkgGpgCheck;
+    if ( ! indeterminate(ZConfig::instance().pkgGpgCheck()) )	return ZConfig::instance().pkgGpgCheck();
+    // no preference: follow gpgCheck and check package if repo signature not available or not checked
+    return gpgCheck() && ( !repoGpgCheck() || !(bool)validRepoSignature() );	// !(bool)TriBool ==> false or indeterminate
+  }
+
+  void RepoInfo::setPkgGpgCheck( TriBool value_r )
+  { _pimpl->_pkgGpgCheck = value_r; }
+
+
+  TriBool RepoInfo::validRepoSignature() const
+  {
+    TriBool ret = _pimpl->internalValidRepoSignature();
+    // keep indeterminate(=unsigned) but invalidate any signature if !repoGpgCheck
+    if ( !indeterminate(ret) && !repoGpgCheck() )
+      ret = false;
+    return ret;
+  }
+
+  void RepoInfo::setValidRepoSignature( TriBool value_r )
+  { _pimpl->internalSetValidRepoSignature( value_r ); }
+
+
+  void RepoInfo::setMirrorListUrl( const Url & url_r )	// Raw
+  { _pimpl->_mirrorListUrl.raw() = url_r; _pimpl->_mirrorListForceMetalink = false; }
+
+  void  RepoInfo::setMetalinkUrl( const Url & url_r )	// Raw
+  { _pimpl->_mirrorListUrl.raw() = url_r; _pimpl->_mirrorListForceMetalink = true; }
+
+  void RepoInfo::setGpgKeyUrl( const Url & url_r )
+  { _pimpl->_gpgKeyUrl.raw() = url_r; }
+
+  void RepoInfo::addBaseUrl( const Url & url_r )
+  {
+    for ( const auto & url : _pimpl->baseUrls().raw() )	// Raw unique!
+      if ( url == url_r )
+	return;
+    _pimpl->baseUrls().raw().push_back( url_r );
+  }
+
+  void RepoInfo::setBaseUrl( const Url & url_r )
+  {
+    _pimpl->baseUrls().raw().clear();
+    _pimpl->baseUrls().raw().push_back( url_r );
   }
 
   void RepoInfo::setPath( const Pathname &path )
@@ -262,9 +356,6 @@ namespace zypp
   void RepoInfo::setTargetDistribution( const std::string & targetDistribution )
   { _pimpl->targetDistro = targetDistribution; }
 
-  bool RepoInfo::gpgCheck() const
-  { return indeterminate(_pimpl->gpgcheck) ? true : (bool)_pimpl->gpgcheck; }
-
   bool RepoInfo::keepPackages() const
   { return indeterminate(_pimpl->keeppackages) ? false : (bool)_pimpl->keeppackages; }
 
@@ -277,23 +368,23 @@ namespace zypp
   repo::RepoType RepoInfo::type() const
   { return _pimpl->type; }
 
-  Url RepoInfo::mirrorListUrl() const
-  { return _pimpl->getmirrorListUrl(); }
+  Url RepoInfo::mirrorListUrl() const			// Variables replaced!
+  { return _pimpl->_mirrorListUrl.transformed(); }
 
-  Url RepoInfo::gpgKeyUrl() const
-  { return _pimpl->gpgkey_url; }
+  Url RepoInfo::rawMirrorListUrl() const		// Raw
+  { return _pimpl->_mirrorListUrl.raw(); }
 
-  std::set<Url> RepoInfo::baseUrls() const
-  {
-    RepoInfo::url_set replaced_urls;
-    for ( url_set::const_iterator it = _pimpl->baseUrls().begin();
-          it != _pimpl->baseUrls().end();
-          ++it )
-    {
-      replaced_urls.insert(_pimpl->replacer(*it));
-    }
-    return replaced_urls;
-  }
+  Url RepoInfo::gpgKeyUrl() const			// Variables replaced!
+  { return _pimpl->_gpgKeyUrl.transformed(); }
+
+  Url RepoInfo::rawGpgKeyUrl() const			// Raw
+  {  return _pimpl->_gpgKeyUrl.raw(); }
+
+  RepoInfo::url_set RepoInfo::baseUrls() const		// Variables replaced!
+  { return _pimpl->baseUrls().transformed(); }
+
+  RepoInfo::url_set RepoInfo::rawBaseUrls() const	// Raw
+  { return _pimpl->baseUrls().raw(); }
 
   Pathname RepoInfo::path() const
   { return _pimpl->path; }
@@ -304,19 +395,14 @@ namespace zypp
   std::string RepoInfo::targetDistribution() const
   { return _pimpl->targetDistro; }
 
+  Url RepoInfo::rawUrl() const
+  { return( _pimpl->baseUrls().empty() ? Url() : *_pimpl->baseUrls().rawBegin() ); }
+
   RepoInfo::urls_const_iterator RepoInfo::baseUrlsBegin() const
-  {
-    return make_transform_iterator( _pimpl->baseUrls().begin(),
-                                    _pimpl->replacer );
-    //return _pimpl->baseUrls.begin();
-  }
+  { return _pimpl->baseUrls().transformedBegin(); }
 
   RepoInfo::urls_const_iterator RepoInfo::baseUrlsEnd() const
-  {
-    //return _pimpl->baseUrls.end();
-    return make_transform_iterator( _pimpl->baseUrls().end(),
-                                    _pimpl->replacer );
-  }
+  { return _pimpl->baseUrls().transformedEnd(); }
 
   RepoInfo::urls_size_type RepoInfo::baseUrlsSize() const
   { return _pimpl->baseUrls().size(); }
@@ -339,9 +425,6 @@ namespace zypp
   bool RepoInfo::hasLicense() const
   {
     Pathname licenseTgz( _pimpl->licenseTgz() );
-    SEC << licenseTgz << endl;
-    SEC << PathInfo(licenseTgz) << endl;
-
     return ! licenseTgz.empty() &&  PathInfo(licenseTgz).isFile();
   }
 
@@ -369,11 +452,14 @@ namespace zypp
         accept = false;
       }
     }
-    MIL << "License for " << this->name() << " has to be accepted: " << (accept?"true":"false" ) << endl;
+    MIL << "License for " << name() << " has to be accepted: " << (accept?"true":"false" ) << endl;
     return accept;
   }
 
   std::string RepoInfo::getLicense( const Locale & lang_r )
+  { return const_cast<const RepoInfo *>(this)->getLicense( lang_r );  }
+
+  std::string RepoInfo::getLicense( const Locale & lang_r ) const
   {
     LocaleSet avlocales( getLicenseLocales() );
     if ( avlocales.empty() )
@@ -453,11 +539,9 @@ namespace zypp
     RepoInfoBase::dumpOn(str);
     if ( _pimpl->baseurl2dump() )
     {
-      for ( urls_const_iterator it = baseUrlsBegin();
-            it != baseUrlsEnd();
-            ++it )
+      for ( const auto & url : _pimpl->baseUrls().raw() )
       {
-        str << "- url         : " << *it << std::endl;
+        str << "- url         : " << url << std::endl;
       }
     }
 
@@ -467,12 +551,21 @@ namespace zypp
 	str << tag_r << value_r << std::endl;
     });
 
-    strif( "- mirrorlist  : ", _pimpl->getmirrorListUrl().asString() );
+    strif( (_pimpl->_mirrorListForceMetalink ? "- metalink    : " : "- mirrorlist  : "), rawMirrorListUrl().asString() );
     strif( "- path        : ", path().asString() );
     str << "- type        : " << type() << std::endl;
     str << "- priority    : " << priority() << std::endl;
-    str << "- gpgcheck    : " << gpgCheck() << std::endl;
-    strif( "- gpgkey      : ", gpgKeyUrl().asString() );
+
+    // Yes No Default(Y) Default(N)
+#define OUTS(T,B) ( indeterminate(T) ? (std::string("D(")+(B?"Y":"N")+")") : ((bool)T?"Y":"N") )
+    str << "- gpgcheck    : " << OUTS(_pimpl->_gpgCheck,gpgCheck())
+                              << " repo" << OUTS(_pimpl->_repoGpgCheck,repoGpgCheck())
+			      << " sig" << asString( validRepoSignature(), "?", "Y", "N" )
+			      << " pkg" << OUTS(_pimpl->_pkgGpgCheck,pkgGpgCheck())
+			      << std::endl;
+#undef OUTS
+
+    strif( "- gpgkey      : ", rawGpgKeyUrl().asString() );
 
     if ( ! indeterminate(_pimpl->keeppackages) )
       str << "- keeppackages: " << keepPackages() << std::endl;
@@ -492,29 +585,36 @@ namespace zypp
     if ( _pimpl->baseurl2dump() )
     {
       str << "baseurl=";
-      for ( url_set::const_iterator it = _pimpl->baseUrls().begin();
-            it != _pimpl->baseUrls().end();
-            ++it )
+      std::string indent;
+      for ( const auto & url : _pimpl->baseUrls().raw() )
       {
-        str << *it << endl;
+        str << indent << url << endl;
+	if ( indent.empty() ) indent = "        ";	// "baseurl="
       }
     }
 
     if ( ! _pimpl->path.empty() )
       str << "path="<< path() << endl;
 
-    if ( ! (_pimpl->getmirrorListUrl().asString().empty()) )
-      str << "mirrorlist=" << _pimpl->getmirrorListUrl() << endl;
+    if ( ! (rawMirrorListUrl().asString().empty()) )
+      str << (_pimpl->_mirrorListForceMetalink ? "metalink=" : "mirrorlist=") << rawMirrorListUrl() << endl;
 
     str << "type=" << type().asString() << endl;
 
     if ( priority() != defaultPriority() )
       str << "priority=" << priority() << endl;
 
-    if (!indeterminate(_pimpl->gpgcheck))
-      str << "gpgcheck=" << (gpgCheck() ? "1" : "0") << endl;
-    if ( ! (gpgKeyUrl().asString().empty()) )
-      str << "gpgkey=" <<gpgKeyUrl() << endl;
+    if ( ! indeterminate(_pimpl->_gpgCheck) )
+      str << "gpgcheck=" << (_pimpl->_gpgCheck ? "1" : "0") << endl;
+
+    if ( ! indeterminate(_pimpl->_repoGpgCheck) )
+      str << "repo_gpgcheck=" << (_pimpl->_repoGpgCheck ? "1" : "0") << endl;
+
+    if ( ! indeterminate(_pimpl->_pkgGpgCheck) )
+      str << "pkg_gpgcheck=" << (_pimpl->_pkgGpgCheck ? "1" : "0") << endl;
+
+    if ( ! (rawGpgKeyUrl().asString().empty()) )
+      str << "gpgkey=" << rawGpgKeyUrl() << endl;
 
     if (!indeterminate(_pimpl->keeppackages))
       str << "keeppackages=" << keepPackages() << endl;
@@ -538,18 +638,19 @@ namespace zypp
       << " priority=\"" << priority() << "\""
       << " enabled=\"" << enabled() << "\""
       << " autorefresh=\"" << autorefresh() << "\""
-      << " gpgcheck=\"" << gpgCheck() << "\"";
+      << " gpgcheck=\"" << gpgCheck() << "\""
+      << " repo_gpgcheck=\"" << repoGpgCheck() << "\""
+      << " pkg_gpgcheck=\"" << pkgGpgCheck() << "\"";
     if (!(tmpstr = gpgKeyUrl().asString()).empty())
       str << " gpgkey=\"" << escape(tmpstr) << "\"";
     if (!(tmpstr = mirrorListUrl().asString()).empty())
-      str << " mirrorlist=\"" << escape(tmpstr) << "\"";
+      str << (_pimpl->_mirrorListForceMetalink ? " metalink=\"" : " mirrorlist=\"") << escape(tmpstr) << "\"";
     str << ">" << endl;
 
     if ( _pimpl->baseurl2dump() )
     {
-      for (RepoInfo::urls_const_iterator urlit = baseUrlsBegin();
-           urlit != baseUrlsEnd(); ++urlit)
-        str << "<url>" << escape(urlit->asString()) << "</url>" << endl;
+      for_( it, baseUrlsBegin(), baseUrlsEnd() )	// !transform iterator replaces variables
+	str << "<url>" << escape((*it).asString()) << "</url>" << endl;
     }
 
     str << "</repo>" << endl;
