@@ -21,6 +21,7 @@
 #include "zypp/base/InputStream.h"
 #include "zypp/base/LogTools.h"
 #include "zypp/base/Gettext.h"
+#include "zypp/base/DefaultIntegral.h"
 #include "zypp/base/Function.h"
 #include "zypp/base/Regex.h"
 #include "zypp/PathInfo.h"
@@ -148,7 +149,7 @@ namespace zypp
     /**
      * \short Simple callback to collect the results
      *
-     * Classes like RepoFileParser call the callback
+     * Classes like RepoFileReader call the callback
      * once per each repo in a file.
      *
      * Passing this functor as callback, you can collect
@@ -223,7 +224,7 @@ namespace zypp
       bool nonroot( geteuid() != 0 );
       if ( nonroot && ! PathInfo(dir).userMayRX() )
       {
-	JobReport::warning( formatNAC(_("Cannot read repo directory ‘%1%’: Permission denied")) % dir );
+	JobReport::warning( formatNAC(_("Cannot read repo directory '%1%': Permission denied")) % dir );
       }
       else
       {
@@ -241,7 +242,7 @@ namespace zypp
 	  {
 	    if ( nonroot && ! PathInfo(*it).userMayR() )
 	    {
-	      JobReport::warning( formatNAC(_("Cannot read repo file ‘%1%’: Permission denied")) % *it );
+	      JobReport::warning( formatNAC(_("Cannot read repo file '%1%': Permission denied")) % *it );
 	    }
 	    else
 	    {
@@ -441,19 +442,62 @@ namespace zypp
       init_knownRepositories();
     }
 
+    ~Impl()
+    {
+      // trigger appdata refresh if some repos change
+      if ( _reposDirty && geteuid() == 0 && ( _options.rootDir.empty() || _options.rootDir == "/" ) )
+      {
+	try {
+	  std::list<Pathname> entries;
+	  filesystem::readdir( entries, _options.pluginsPath/"appdata", false );
+	  if ( ! entries.empty() )
+	  {
+	    ExternalProgram::Arguments cmd;
+	    cmd.push_back( "<" );		// discard stdin
+	    cmd.push_back( ">" );		// discard stdout
+	    cmd.push_back( "PROGRAM" );		// [2] - fix index below if changing!
+	    for ( const auto & rinfo : repos() )
+	    {
+	      if ( ! rinfo.enabled() )
+		continue;
+	      cmd.push_back( "-R" );
+	      cmd.push_back( rinfo.alias() );
+	      cmd.push_back( "-t" );
+	      cmd.push_back( rinfo.type().asString() );
+	      cmd.push_back( "-p" );
+	      cmd.push_back( rinfo.metadataPath().asString() );
+	    }
+
+	    for_( it, entries.begin(), entries.end() )
+	    {
+	      PathInfo pi( *it );
+	      //DBG << "/tmp/xx ->" << pi << endl;
+	      if ( pi.isFile() && pi.userMayRX() )
+	      {
+		// trigger plugin
+		cmd[2] = pi.asString();		// [2] - PROGRAM
+		ExternalProgram prog( cmd, ExternalProgram::Stderr_To_Stdout );
+	      }
+	    }
+	  }
+	}
+	catch (...) {}	// no throw in dtor
+      }
+    }
+
   public:
-    bool repoEmpty() const		{ return _repos.empty(); }
-    RepoSizeType repoSize() const	{ return _repos.size(); }
-    RepoConstIterator repoBegin() const	{ return _repos.begin(); }
-    RepoConstIterator repoEnd() const	{ return _repos.end(); }
+    bool repoEmpty() const		{ return repos().empty(); }
+    RepoSizeType repoSize() const	{ return repos().size(); }
+    RepoConstIterator repoBegin() const	{ return repos().begin(); }
+    RepoConstIterator repoEnd() const	{ return repos().end(); }
 
     bool hasRepo( const std::string & alias ) const
-    { return foundAliasIn( alias, _repos ); }
+    { return foundAliasIn( alias, repos() ); }
 
     RepoInfo getRepo( const std::string & alias ) const
     {
-      RepoConstIterator it( findAlias( alias, _repos ) );
-      return it == _repos.end() ? RepoInfo::noRepo : *it;
+      RepoConstIterator it( findAlias( alias, repos() ) );
+      return it == repos().end() ? RepoInfo::noRepo : *it;
     }
 
   public:
@@ -558,8 +602,8 @@ namespace zypp
     void getRepositoriesInService( const std::string & alias, OutputIterator out ) const
     {
       MatchServiceAlias filter( alias );
-      std::copy( boost::make_filter_iterator( filter, _repos.begin(), _repos.end() ),
-                 boost::make_filter_iterator( filter, _repos.end(), _repos.end() ),
+      std::copy( boost::make_filter_iterator( filter, repos().begin(), repos().end() ),
+                 boost::make_filter_iterator( filter, repos().end(), repos().end() ),
                  out);
     }
 
@@ -567,11 +611,16 @@ namespace zypp
     void init_knownServices();
     void init_knownRepositories();
 
+    const RepoSet & repos() const { return _reposX; }
+    RepoSet & reposManip()        { if ( ! _reposDirty ) _reposDirty = true; return _reposX; }
+
   private:
     RepoManagerOptions	_options;
-    RepoSet 		_repos;
+    RepoSet 		_reposX;
     ServiceSet		_services;
 
+    DefaultIntegral<bool,false> _reposDirty;
+
   private:
     friend Impl * rwcowClone<Impl>( const Impl * rhs );
     /** clone for RWCOW_pointer */
@@ -703,7 +752,7 @@ namespace zypp
 	// set the downloaded packages path for the repo
 	repoInfo.setPackagesPath( packagescache_path_for_repoinfo(_options, repoInfo) );
 	// remember it
-        _repos.insert( repoInfo );
+        _reposX.insert( repoInfo );	// direct access via _reposX in ctor! no reposManip.
 
 	// detect orphaned repos belonging to a deleted service
 	const std::string & serviceAlias( repoInfo.service() );
@@ -963,6 +1012,9 @@ namespace zypp
 					"Valid metadata not found at specified URLs",
 					info.baseUrlsSize() ) );
 
+    // Suppress (interactive) media::MediaChangeReport if we in have multiple basurls (>1)
+    media::ScopedDisableMediaChangeReport guard( info.baseUrlsSize() > 1 );
+
     // try urls one by one
     for ( RepoInfo::urls_const_iterator it = info.baseUrlsBegin(); it != info.baseUrlsEnd(); ++it )
     {
@@ -1063,6 +1115,7 @@ namespace zypp
         // ok we have the metadata, now exchange
         // the contents
 	filesystem::exchange( tmpdir.path(), mediarootpath );
+	reposManip();	// remember to trigger appdata refresh
 
         // we are done.
         return;
@@ -1240,6 +1293,7 @@ namespace zypp
 
         // We keep it.
         guard.resetDispose();
+	sat::updateSolvFileIndex( solvfile );	// content digest for zypper bash completion
       }
       break;
       default:
@@ -1454,7 +1508,7 @@ namespace zypp
     MIL << "Try adding repo " << info << endl;
 
     RepoInfo tosave = info;
-    if ( _repos.find(tosave) != _repos.end() )
+    if ( repos().find(tosave) != repos().end() )
       ZYPP_THROW(RepoAlreadyExistsException(info));
 
     // check the first url for now
@@ -1501,7 +1555,7 @@ namespace zypp
       oinfo.setMetadataPath( metadataPath( tosave ) );
       oinfo.setPackagesPath( packagesPath( tosave ) );
     }
-    _repos.insert(tosave);
+    reposManip().insert(tosave);
 
     progress.set(90);
 
@@ -1579,7 +1633,7 @@ namespace zypp
       MIL << "Saving " << (*it).alias() << endl;
       it->setFilepath(repofile.asString());
       it->dumpAsIniOn(file);
-      _repos.insert(*it);
+      reposManip().insert(*it);
 
       HistoryLog(_options.rootDir).addRepository(*it);
     }
@@ -1627,7 +1681,7 @@ namespace zypp
             // TranslatorExplanation '%s' is a filename
             ZYPP_THROW(RepoException( todelete, str::form( _("Can't delete '%s'"), todelete.filepath().c_str() )));
           }
-          MIL << todelete.alias() << " sucessfully deleted." << endl;
+          MIL << todelete.alias() << " successfully deleted." << endl;
         }
         else
         {
@@ -1663,8 +1717,8 @@ namespace zypp
         // now delete metadata (#301037)
         cleanMetadata( todelete, mSubprogrcv );
 	cleanPackages( todelete, pSubprogrcv );
-        _repos.erase(todelete);
-        MIL << todelete.alias() << " sucessfully deleted." << endl;
+        reposManip().erase(todelete);
+        MIL << todelete.alias() << " successfully deleted." << endl;
         HistoryLog(_options.rootDir).removeRepository(todelete);
         return;
       } // else filepath is empty
@@ -1723,8 +1777,8 @@ namespace zypp
       }
 
       newinfo.setFilepath(toedit.filepath());
-      _repos.erase(toedit);
-      _repos.insert(newinfo);
+      reposManip().erase(toedit);
+      reposManip().insert(newinfo);
       HistoryLog(_options.rootDir).modifyRepository(toedit, newinfo);
       MIL << "repo " << alias << " modified" << endl;
     }
@@ -1734,8 +1788,8 @@ namespace zypp
 
   RepoInfo RepoManager::Impl::getRepositoryInfo( const std::string & alias, const ProgressData::ReceiverFnc & progressrcv )
   {
-    RepoConstIterator it( findAlias( alias, _repos ) );
-    if ( it != _repos.end() )
+    RepoConstIterator it( findAlias( alias, repos() ) );
+    if ( it != repos().end() )
       return *it;
     RepoInfo info;
     info.setAlias( alias );
@@ -1816,7 +1870,7 @@ namespace zypp
         // TranslatorExplanation '%s' is a filename
         ZYPP_THROW(ServiceException( service, str::form( _("Can't delete '%s'"), location.c_str() ) ));
       }
-      MIL << alias << " sucessfully deleted." << endl;
+      MIL << alias << " successfully deleted." << endl;
     }
     else
     {
@@ -1835,7 +1889,7 @@ namespace zypp
           it->dumpAsIniOn(file);
       }
 
-      MIL << alias << " sucessfully deleted from file " << location <<  endl;
+      MIL << alias << " successfully deleted from file " << location <<  endl;
     }
 
     // now remove all repositories added by this service
@@ -1935,11 +1989,11 @@ namespace zypp
       // if the repo url was not set by the repoindex parser, set service's url
       Url url;
       if ( it->baseUrlsEmpty() )
-        url = service.url();
+        url = service.rawUrl();
       else
       {
         // service repo can contain only one URL now, so no need to iterate.
-        url = *it->baseUrlsBegin();
+        url = it->rawUrl();	// raw!
       }
 
       // libzypp currently has problem with separate url + path handling
@@ -2089,6 +2143,14 @@ namespace zypp
 
 	// all other attributes follow the service request:
 
+	// changed name (raw!)
+	if ( oldRepo->rawName() != it->rawName() )
+	{
+	  DBG << "Service repo " << it->alias() << " gets new NAME " << it->rawName() << endl;
+	  oldRepo->setName( it->rawName() );
+	  oldRepoModified = true;
+	}
+
 	// changed autorefresh
 	if ( oldRepo->autorefresh() != it->autorefresh() )
 	{
@@ -2107,10 +2169,10 @@ namespace zypp
 
         // changed url?
         // service repo can contain only one URL now, so no need to iterate.
-        if ( oldRepo->url() != it->url() )
+        if ( oldRepo->rawUrl() != it->rawUrl() )
         {
-          DBG << "Service repo " << it->alias() << " gets new URL " << it->url() << endl;
-          oldRepo->setBaseUrl( it->url() );
+          DBG << "Service repo " << it->alias() << " gets new URL " << it->rawUrl() << endl;
+          oldRepo->setBaseUrl( it->rawUrl() );
           oldRepoModified = true;
         }
 
