@@ -305,6 +305,9 @@ namespace zypp
         , download_max_silent_tries	( 5 )
         , download_transfer_timeout	( 180 )
         , commit_downloadMode		( DownloadDefault )
+	, gpgCheck			( true )
+	, repoGpgCheck			( indeterminate )
+	, pkgGpgCheck			( indeterminate )
         , solver_onlyRequires		( false )
         , solver_allowVendorChange	( false )
         , solver_cleandepsOnRemove	( false )
@@ -440,6 +443,18 @@ namespace zypp
                 {
                   commit_downloadMode.set( deserializeDownloadMode( value ) );
                 }
+                else if ( entry == "gpgcheck" )
+		{
+		  gpgCheck.set( str::strToBool( value, gpgCheck ) );
+		}
+		else if ( entry == "repo_gpgcheck" )
+		{
+		  repoGpgCheck.set( str::strToTriBool( value ) );
+		}
+		else if ( entry == "pkg_gpgcheck" )
+		{
+		  pkgGpgCheck.set( str::strToTriBool( value ) );
+		}
                 else if ( entry == "vendordir" )
                 {
                   cfg_vendor_path = Pathname(value);
@@ -472,6 +487,10 @@ namespace zypp
                 {
                   solver_checkSystemFile = Pathname(value);
                 }
+                else if ( entry == "solver.checkSystemFileDir" )
+                {
+                  solver_checkSystemFileDir = Pathname(value);
+                }
                 else if ( entry == "multiversion" )
                 {
                   str::split( value, inserter( _multiversion, _multiversion.end() ), ", \t" );
@@ -585,6 +604,10 @@ namespace zypp
 
     Option<DownloadMode> commit_downloadMode;
 
+    Option<bool>	gpgCheck;
+    Option<TriBool>	repoGpgCheck;
+    Option<TriBool>	pkgGpgCheck;
+
     Option<bool>	solver_onlyRequires;
     Option<bool>	solver_allowVendorChange;
     Option<bool>	solver_cleandepsOnRemove;
@@ -592,6 +615,7 @@ namespace zypp
     DefaultOption<bool> solverUpgradeRemoveDroppedPackages;
 
     Pathname solver_checkSystemFile;
+    Pathname solver_checkSystemFileDir;
 
     std::set<std::string> &		multiversion()		{ return getMultiversion(); }
     const std::set<std::string> &	multiversion() const	{ return getMultiversion(); }
@@ -866,6 +890,15 @@ namespace zypp
   DownloadMode ZConfig::commit_downloadMode() const
   { return _pimpl->commit_downloadMode; }
 
+  bool ZConfig::gpgCheck() const
+  { return _pimpl->gpgCheck; }
+
+  TriBool ZConfig::repoGpgCheck() const
+  { return _pimpl->repoGpgCheck; }
+
+  TriBool ZConfig::pkgGpgCheck() const
+  { return _pimpl->pkgGpgCheck; }
+
   bool ZConfig::solver_onlyRequires() const
   { return _pimpl->solver_onlyRequires; }
 
@@ -879,6 +912,10 @@ namespace zypp
   { return ( _pimpl->solver_checkSystemFile.empty()
       ? (configPath()/"systemCheck") : _pimpl->solver_checkSystemFile ); }
 
+  Pathname ZConfig::solver_checkSystemFileDir() const
+  { return ( _pimpl->solver_checkSystemFileDir.empty()
+      ? (configPath()/"systemCheck.d") : _pimpl->solver_checkSystemFileDir ); }
+
   unsigned ZConfig::solver_upgradeTestcasesToKeep() const
   { return _pimpl->solver_upgradeTestcasesToKeep; }
 
