@@ -157,23 +157,6 @@ namespace zypp
       virtual void finishDeltaApply()
       {}
 
-      /** \deprecated Unused since 2008 */
-      virtual ZYPP_DEPRECATED void startPatchDownload( const Pathname & /*filename*/, const ByteCount & /*downloadsize*/ )
-      {}
-
-      /** \deprecated Unused since 2008 */
-      virtual ZYPP_DEPRECATED bool progressPatchDownload( int /*value*/ )
-      { return true; }
-
-      /** \deprecated Unused since 2008 */
-      virtual ZYPP_DEPRECATED void problemPatchDownload( const std::string &/*description*/ )
-      {}
-
-      /** \deprecated Unused since 2008 */
-      virtual ZYPP_DEPRECATED void finishPatchDownload()
-      {}
-
-
       // return false if the download should be aborted right now
       virtual bool progress(int /*value*/, Resolvable::constPtr /*resolvable_ptr*/)
       { return true; }
@@ -184,6 +167,22 @@ namespace zypp
 	, const std::string &/*description*/
       ) { return ABORT; }
 
+
+      /** Detail information about the result of a performed pkgGpgCheck.
+       *
+       * Userdata sent:
+       * \param "Package"	Package::constPtr of the package
+       * \param "Localpath"	Pathname to downloaded package on disk
+       * \param "CheckPackageResult"	RpmDb::CheckPackageResult of signature check
+       * \param "CheckPackageDetail"	RpmDb::CheckPackageDetail logmessages of rpm signature check
+       *
+       *  Userdata accepted:
+       * \param "Action"	DownloadResolvableReport::Action user advice how to behave on error (ABORT).
+       *                        If you set just an empty value here, a default probelm report will be triggered.
+       */
+      virtual void pkgGpgCheck( const UserData & userData_r = UserData() )
+      {}
+
       virtual void finish(Resolvable::constPtr /*resolvable_ptr*/
         , Error /*error*/
         , const std::string &/*reason*/
@@ -337,6 +336,19 @@ namespace zypp
       ) { return ABORT; }
     };
 
+    ///////////////////////////////////////////////////////////////////
+    /// \class ScopedDisableMediaChangeReport
+    /// \brief Temporarily disable MediaChangeReport
+    /// Sometimes helpful to suppress interactive messages connected to
+    /// MediaChangeReport while fallback URLs are avaialble.
+    struct ScopedDisableMediaChangeReport
+    {
+      /** Disbale MediaChangeReport if \a condition_r is \c true.*/
+      ScopedDisableMediaChangeReport( bool condition_r = true );
+    private:
+      shared_ptr<callback::TempConnect<media::MediaChangeReport> > _guard;
+    };
+
     // progress for downloading a file
     struct DownloadProgressReport : public callback::ReportBase
     {
