@@ -158,6 +158,7 @@ SATResolver::dumpOn( std::ostream & os ) const
         os << "  distupgrade_removeunsupported	= " << _distupgrade_removeunsupported << endl;
 	os << "  solveSrcPackages	= "	<< _solveSrcPackages << endl;
 	os << "  cleandepsOnRemove	= "	<< _cleandepsOnRemove << endl;
+        os << "  fixsystem		= "	<< _fixsystem << endl;
     } else {
 	os << "<NULL>";
     }
@@ -177,7 +178,7 @@ SATResolver::SATResolver (const ResPool & pool, Pool *SATPool)
     , _allowuninstall(false)
     , _updatesystem(false)
     , _noupdateprovide(false)
-    , _dosplitprovides(false)
+    , _dosplitprovides(true)
     , _onlyRequires(ZConfig::instance().solver_onlyRequires())
     , _ignorealreadyrecommended(true)
     , _distupgrade(false)
@@ -1200,7 +1201,7 @@ SATResolver::problems ()
 				if (poolItem) {
 				    if (pool->installed && s.get()->repo == pool->installed) {
 					problemSolution->addSingleAction (poolItem, REMOVE);
-					string description = str::form (_("do not keep %s installed"),  s.asString().c_str() );
+					string description = str::form (_("remove lock to allow removal of %s"),  s.asString().c_str() );
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    } else {
@@ -1225,7 +1226,7 @@ SATResolver::problems ()
 					problemSolution->addDescription (description);
 				    } else {
 					problemSolution->addSingleAction (poolItem, UNLOCK);
-					string description = str::form (_("do not forbid installation of %s"), itemToString( poolItem ).c_str());
+					string description = str::form (_("remove lock to allow installation of %s"), itemToString( poolItem ).c_str());
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    }
