@@ -1059,7 +1059,7 @@ namespace zypp
 
         cmd << "  > '" << tmpsolv.path() << "'";
 
-        MIL << "Executing: " << cmd << endl;
+        MIL << "Executing: " << cmd.str() << endl;
         ExternalProgram prog( cmd.str(), ExternalProgram::Stderr_To_Stdout );
 
         cmd << endl;
@@ -1086,6 +1086,7 @@ namespace zypp
 
         // We keep it.
         guard.resetDispose();
+	sat::updateSolvFileIndex( rpmsolv );	// content digest for zypper bash completion
 
 	// Finally send notification to plugins
 	// NOTE: quick hack looking for spacewalk plugin only
@@ -1170,6 +1171,7 @@ namespace zypp
 
         system.addSolv( rpmsolv );
       }
+      sat::Pool::instance().rootDir( _root );
 
       // (Re)Load the requested locales et al.
       // If the requested locales are empty, we leave the pool untouched
@@ -1863,6 +1865,12 @@ namespace zypp
     std::string TargetImpl::targetDistributionRelease( const Pathname & root_r )
     { return baseproductdata( staticGuessRoot(root_r) ).registerRelease();}
 
+    std::string TargetImpl::targetDistributionFlavor() const
+    { return baseproductdata( _root ).registerFlavor(); }
+    // static version:
+    std::string TargetImpl::targetDistributionFlavor( const Pathname & root_r )
+    { return baseproductdata( staticGuessRoot(root_r) ).registerFlavor();}
+
     Target::DistributionLabel TargetImpl::distributionLabel() const
     {
       Target::DistributionLabel ret;
